@@ -29,12 +29,25 @@
 #include "qgsmultipolygon.h"
 #include "qgsgeos.h"
 #include "qgssimplelinematerialsettings.h"
-
+#include "qgspolygon.h"
 #include "qgsphongtexturedmaterialsettings.h"
 
 #include <Qt3DExtras/QPhongMaterial>
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 #include <Qt3DRender/QAttribute>
 #include <Qt3DRender/QBuffer>
+
+typedef Qt3DRender::QAttribute Qt3DQAttribute;
+typedef Qt3DRender::QBuffer Qt3DQBuffer;
+typedef Qt3DRender::QGeometry Qt3DQGeometry;
+#else
+#include <Qt3DCore/QAttribute>
+#include <Qt3DCore/QBuffer>
+
+typedef Qt3DCore::QAttribute Qt3DQAttribute;
+typedef Qt3DCore::QBuffer Qt3DQBuffer;
+typedef Qt3DCore::QGeometry Qt3DQGeometry;
+#endif
 #include <Qt3DRender/QGeometryRenderer>
 
 /// @cond PRIVATE
@@ -50,7 +63,7 @@ class QgsBufferedLine3DSymbolHandler : public QgsFeature3DHandler
       , mSelectedIds( selectedIds ) {}
 
     bool prepare( const Qgs3DRenderContext &context, QSet<QString> &attributeNames ) override;
-    void processFeature( QgsFeature &feature, const Qgs3DRenderContext &context ) override;
+    void processFeature( const QgsFeature &feature, const Qgs3DRenderContext &context ) override;
     void finalize( Qt3DCore::QEntity *parent, const Qgs3DRenderContext &context ) override;
 
   private:
@@ -97,7 +110,7 @@ bool QgsBufferedLine3DSymbolHandler::prepare( const Qgs3DRenderContext &context,
   return true;
 }
 
-void QgsBufferedLine3DSymbolHandler::processFeature( QgsFeature &f, const Qgs3DRenderContext &context )
+void QgsBufferedLine3DSymbolHandler::processFeature( const QgsFeature &f, const Qgs3DRenderContext &context )
 {
   if ( f.geometry().isNull() )
     return;
@@ -105,20 +118,22 @@ void QgsBufferedLine3DSymbolHandler::processFeature( QgsFeature &f, const Qgs3DR
   LineData &out = mSelectedIds.contains( f.id() ) ? outSelected : outNormal;
 
   QgsGeometry geom = f.geometry();
+  const QgsAbstractGeometry *g = geom.constGet()->simplifiedTypeRef();
 
   // segmentize curved geometries if necessary
-  if ( QgsWkbTypes::isCurvedType( geom.constGet()->wkbType() ) )
-    geom = QgsGeometry( geom.constGet()->segmentize() );
-
-  const QgsAbstractGeometry *g = geom.constGet();
+  if ( QgsWkbTypes::isCurvedType( g->wkbType() ) )
+  {
+    geom = QgsGeometry( g->segmentize() );
+    g = geom.constGet()->simplifiedTypeRef();
+  }
 
   // TODO: configurable
   const int nSegments = 4;
-  const QgsGeometry::EndCapStyle endCapStyle = QgsGeometry::CapRound;
-  const QgsGeometry::JoinStyle joinStyle = QgsGeometry::JoinStyleRound;
+  const Qgis::EndCapStyle endCapStyle = Qgis::EndCapStyle::Round;
+  const Qgis::JoinStyle joinStyle = Qgis::JoinStyle::Round;
   const double mitreLimit = 0;
 
-  QgsGeos engine( g );
+  const QgsGeos engine( g );
 
   double width = mSymbol->width();
   if ( qgsDoubleNear( width, 0 ) )
@@ -147,6 +162,7 @@ void QgsBufferedLine3DSymbolHandler::processFeature( QgsFeature &f, const Qgs3DR
     }
     delete buffered;
   }
+  mFeatureCount++;
 }
 
 void QgsBufferedLine3DSymbolHandler::processPolygon( QgsPolygon *polyBuffered, QgsFeatureId fid, float height, float extrusionHeight, const Qgs3DRenderContext &context, LineData &out )
@@ -154,7 +170,7 @@ void QgsBufferedLine3DSymbolHandler::processPolygon( QgsPolygon *polyBuffered, Q
   Qgs3DUtils::clampAltitudes( polyBuffered, mSymbol->altitudeClamping(), mSymbol->altitudeBinding(), height, context.map() );
 
   Q_ASSERT( out.tessellator->dataVerticesCount() % 3 == 0 );
-  uint startingTriangleIndex = static_cast<uint>( out.tessellator->dataVerticesCount() / 3 );
+  const uint startingTriangleIndex = static_cast<uint>( out.tessellator->dataVerticesCount() / 3 );
   out.triangleIndexStartingIndices.append( startingTriangleIndex );
   out.triangleIndexFids.append( fid );
   out.tessellator->addPolygon( *polyBuffered, extrusionHeight );
@@ -183,8 +199,8 @@ void QgsBufferedLine3DSymbolHandler::makeEntity( Qt3DCore::QEntity *parent, cons
   Qt3DRender::QMaterial *mat = mSymbol->material()->toMaterial( QgsMaterialSettingsRenderingTechnique::Triangles, materialContext );
 
   // extract vertex buffer data from tessellator
-  QByteArray data( ( const char * )out.tessellator->data().constData(), out.tessellator->data().count() * sizeof( float ) );
-  int nVerts = data.count() / out.tessellator->stride();
+  const QByteArray data( ( const char * )out.tessellator->data().constData(), out.tessellator->data().count() * sizeof( float ) );
+  const int nVerts = data.count() / out.tessellator->stride();
 
   const QgsPhongTexturedMaterialSettings *texturedMaterialSettings = dynamic_cast< const QgsPhongTexturedMaterialSettings * >( mSymbol->material() );
 
@@ -222,7 +238,7 @@ class QgsSimpleLine3DSymbolHandler : public QgsFeature3DHandler
     }
 
     bool prepare( const Qgs3DRenderContext &context, QSet<QString> &attributeNames ) override;
-    void processFeature( QgsFeature &feature, const Qgs3DRenderContext &context ) override;
+    void processFeature( const QgsFeature &feature, const Qgs3DRenderContext &context ) override;
     void finalize( Qt3DCore::QEntity *parent, const Qgs3DRenderContext &context ) override;
 
   private:
@@ -252,7 +268,7 @@ bool QgsSimpleLine3DSymbolHandler::prepare( const Qgs3DRenderContext &context, Q
   return true;
 }
 
-void QgsSimpleLine3DSymbolHandler::processFeature( QgsFeature &f, const Qgs3DRenderContext &context )
+void QgsSimpleLine3DSymbolHandler::processFeature( const QgsFeature &f, const Qgs3DRenderContext &context )
 {
   Q_UNUSED( context )
   if ( f.geometry().isNull() )
@@ -260,7 +276,7 @@ void QgsSimpleLine3DSymbolHandler::processFeature( QgsFeature &f, const Qgs3DRen
 
   QgsLineVertexData &out = mSelectedIds.contains( f.id() ) ? outSelected : outNormal;
 
-  QgsGeometry geom = f.geometry();
+  const QgsGeometry geom = f.geometry();
   const QgsAbstractGeometry *g = geom.constGet();
   if ( const QgsLineString *ls = qgsgeometry_cast<const QgsLineString *>( g ) )
   {
@@ -274,6 +290,7 @@ void QgsSimpleLine3DSymbolHandler::processFeature( QgsFeature &f, const Qgs3DRen
       out.addLineString( *ls );
     }
   }
+  mFeatureCount++;
 }
 
 void QgsSimpleLine3DSymbolHandler::finalize( Qt3DCore::QEntity *parent, const Qgs3DRenderContext &context )
@@ -303,7 +320,7 @@ void QgsSimpleLine3DSymbolHandler::makeEntity( Qt3DCore::QEntity *parent, const
 
   Qt3DCore::QEntity *entity = new Qt3DCore::QEntity;
 
-  Qt3DRender::QGeometry *geom = out.createGeometry( entity );
+  Qt3DQGeometry *geom = out.createGeometry( entity );
 
   Qt3DRender::QGeometryRenderer *renderer = new Qt3DRender::QGeometryRenderer;
   renderer->setPrimitiveType( Qt3DRender::QGeometryRenderer::LineStrip );
@@ -333,7 +350,7 @@ class QgsThickLine3DSymbolHandler : public QgsFeature3DHandler
     }
 
     bool prepare( const Qgs3DRenderContext &context, QSet<QString> &attributeNames ) override;
-    void processFeature( QgsFeature &feature, const Qgs3DRenderContext &context ) override;
+    void processFeature( const QgsFeature &feature, const Qgs3DRenderContext &context ) override;
     void finalize( Qt3DCore::QEntity *parent, const Qgs3DRenderContext &context ) override;
 
   private:
@@ -366,7 +383,7 @@ bool QgsThickLine3DSymbolHandler::prepare( const Qgs3DRenderContext &context, QS
   return true;
 }
 
-void QgsThickLine3DSymbolHandler::processFeature( QgsFeature &f, const Qgs3DRenderContext &context )
+void QgsThickLine3DSymbolHandler::processFeature( const QgsFeature &f, const Qgs3DRenderContext &context )
 {
   Q_UNUSED( context )
   if ( f.geometry().isNull() )
@@ -375,7 +392,15 @@ void QgsThickLine3DSymbolHandler::processFeature( QgsFeature &f, const Qgs3DRend
   QgsLineVertexData &out = mSelectedIds.contains( f.id() ) ? outSelected : outNormal;
 
   QgsGeometry geom = f.geometry();
-  const QgsAbstractGeometry *g = geom.constGet();
+  const QgsAbstractGeometry *g = geom.constGet()->simplifiedTypeRef();
+
+  // segmentize curved geometries if necessary
+  if ( QgsWkbTypes::isCurvedType( g->wkbType() ) )
+  {
+    geom = QgsGeometry( g->segmentize() );
+    g = geom.constGet()->simplifiedTypeRef();
+  }
+
   if ( const QgsLineString *ls = qgsgeometry_cast<const QgsLineString *>( g ) )
   {
     out.addLineString( *ls );
@@ -388,6 +413,7 @@ void QgsThickLine3DSymbolHandler::processFeature( QgsFeature &f, const Qgs3DRend
       out.addLineString( *ls );
     }
   }
+  mFeatureCount++;
 }
 
 void QgsThickLine3DSymbolHandler::finalize( Qt3DCore::QEntity *parent, const Qgs3DRenderContext &context )
@@ -413,7 +439,7 @@ void QgsThickLine3DSymbolHandler::makeEntity( Qt3DCore::QEntity *parent, const Q
   Qt3DRender::QMaterial *mat = mSymbol->material()->toMaterial( QgsMaterialSettingsRenderingTechnique::Lines, materialContext );
   if ( !mat )
   {
-    QgsSimpleLineMaterialSettings defaultMaterial;
+    const QgsSimpleLineMaterialSettings defaultMaterial;
     mat = defaultMaterial.toMaterial( QgsMaterialSettingsRenderingTechnique::Lines, materialContext );
   }
 
