@@ -36,6 +36,41 @@
 #include "qgsgdalutils.h"
 #endif
 
+
+//
+// global callback function
+//
+int CPL_STDCALL GdalProgressCallback( double dfComplete,
+                                      const char *pszMessage,
+                                      void *pProgressArg )
+{
+  Q_UNUSED( pszMessage )
+
+  static double sDfLastComplete = -1.0;
+
+  QgsFeedback *feedback = static_cast<QgsFeedback *>( pProgressArg );
+
+  if ( sDfLastComplete > dfComplete )
+  {
+    if ( sDfLastComplete >= 1.0 )
+      sDfLastComplete = -1.0;
+    else
+      sDfLastComplete = dfComplete;
+  }
+
+  if ( std::floor( sDfLastComplete * 10 ) != std::floor( dfComplete * 10 ) )
+  {
+    if ( feedback )
+      feedback->setProgress( dfComplete * 100 );
+  }
+  sDfLastComplete = dfComplete;
+
+  if ( feedback && feedback->isCanceled() )
+    return false;
+
+  return true;
+}
+
 QgsRasterCalculator::QgsRasterCalculator( const QString &formulaString, const QString &outputFile, const QString &outputFormat, const QgsRectangle &outputExtent, int nOutputColumns, int nOutputRows, const QVector<QgsRasterCalculatorEntry> &rasterEntries, const QgsCoordinateTransformContext &transformContext )
   : mFormulaString( formulaString )
   , mOutputFile( outputFile )
@@ -93,8 +128,8 @@ QgsRasterCalculator::QgsRasterCalculator( const QString &formulaString, const QS
   , mNumOutputColumns( nOutputColumns )
   , mNumOutputRows( nOutputRows )
   , mRasterEntries( rasterEntries )
+  , mTransformContext( QgsProject::instance()->transformContext() )
 {
-  mTransformContext = QgsProject::instance()->transformContext();
 }
 
 QgsRasterCalculator::Result QgsRasterCalculator::processCalculation( QgsFeedback *feedback )
@@ -110,7 +145,7 @@ QgsRasterCalculator::Result QgsRasterCalculator::processCalculation( QgsFeedback
   }
 
   // Check input layers and bands
-  for ( const auto &entry : qgis::as_const( mRasterEntries ) )
+  for ( const auto &entry : std::as_const( mRasterEntries ) )
   {
     if ( !entry.raster ) // no raster layer in entry
     {
@@ -131,7 +166,6 @@ QgsRasterCalculator::Result QgsRasterCalculator::processCalculation( QgsFeedback
 
 #ifdef HAVE_OPENCL
   // Check for matrix nodes, GPU implementation does not support them
-  QList<const QgsRasterCalcNode *> nodeList;
   if ( QgsOpenClUtils::enabled() && QgsOpenClUtils::available() && ! requiresMatrix )
   {
     return processCalculationGPU( std::move( calcNode ), feedback );
@@ -173,12 +207,12 @@ QgsRasterCalculator::Result QgsRasterCalculator::processCalculation( QgsFeedback
       layerRef.chop( 1 );
       if ( ! inputBlocks.count( layerRef ) )
       {
-        for ( const QgsRasterCalculatorEntry &ref : qgis::as_const( mRasterEntries ) )
+        for ( const QgsRasterCalculatorEntry &ref : std::as_const( mRasterEntries ) )
         {
           if ( ref.ref == layerRef )
           {
             uniqueRasterEntries[layerRef] = ref;
-            inputBlocks[layerRef ] = qgis::make_unique<QgsRasterBlock>();
+            inputBlocks[layerRef ] = std::make_unique<QgsRasterBlock>();
           }
         }
       }
@@ -354,6 +388,9 @@ QgsRasterCalculator::Result QgsRasterCalculator::processCalculation( QgsFeedback
     gdal::fast_delete_and_close( outputDataset, outputDriver, mOutputFile );
     return Canceled;
   }
+
+  GDALComputeRasterStatistics( outputRasterBand, true, nullptr, nullptr, nullptr, nullptr, GdalProgressCallback, feedback );
+
   return Success;
 }
 
@@ -365,7 +402,7 @@ QgsRasterCalculator::Result QgsRasterCalculator::processCalculationGPU( std::uni
 
   QList<const QgsRasterCalcNode *> nodeList( calcNode->findNodes( QgsRasterCalcNode::Type::tRasterRef ) );
   QSet<QString> capturedTexts;
-  for ( const auto &r : qgis::as_const( nodeList ) )
+  for ( const auto &r : std::as_const( nodeList ) )
   {
     QString s( r->toString().remove( 0, 1 ) );
     s.chop( 1 );
@@ -399,7 +436,7 @@ QgsRasterCalculator::Result QgsRasterCalculator::processCalculationGPU( std::uni
     LayerRef entry;
     entry.name = r;
     entry.band = parts[1].toInt( &ok );
-    for ( const auto &ref : mRasterEntries )
+    for ( const auto &ref : std::as_const( mRasterEntries ) )
     {
       if ( ref.ref == entry.name )
         entry.layer = ref.raster;
@@ -459,8 +496,7 @@ QgsRasterCalculator::Result QgsRasterCalculator::processCalculationGPU( std::uni
     {
       cExpression.replace( QStringLiteral( "\"%1\"" ).arg( ref.name ), QStringLiteral( "%1[i]" ).arg( ref.varName ) );
       inputArgs.append( QStringLiteral( "__global %1 *%2" )
-                        .arg( ref.typeName )
-                        .arg( ref.varName ) );
+                        .arg( ref.typeName, ref.varName ) );
       inputBuffers.push_back( cl::Buffer( ctx, CL_MEM_READ_ONLY, ref.bufferSize, nullptr, nullptr ) );
     }
 
@@ -490,20 +526,20 @@ QgsRasterCalculator::Result QgsRasterCalculator::processCalculationGPU( std::uni
     QStringList inputNoDataCheck;
     for ( const auto &ref : inputRefs )
     {
-      inputDesc.append( QStringLiteral( "  // %1 = %2" ).arg( ref.varName ).arg( ref.name ) );
+      inputDesc.append( QStringLiteral( "  // %1 = %2" ).arg( ref.varName, ref.name ) );
       if ( ref.layer->dataProvider()->sourceHasNoDataValue( ref.band ) )
       {
-        inputNoDataCheck.append( QStringLiteral( "(float) %1[i] == (float) %2" ).arg( ref.varName ).arg( ref.layer->dataProvider()->sourceNoDataValue( ref.band ) ) );
+        inputNoDataCheck.append( QStringLiteral( "(float) %1[i] == (float) %2" ).arg( ref.varName, QString::number( ref.layer->dataProvider()->sourceNoDataValue( ref.band ), 'g', 10 ) ) );
       }
     }
 
-    programTemplate = programTemplate.replace( QLatin1String( "##INPUT_NODATA_CHECK##" ), inputNoDataCheck.isEmpty() ? QStringLiteral( "false" ) : inputNoDataCheck.join( QStringLiteral( " || " ) ) );
+    programTemplate = programTemplate.replace( QLatin1String( "##INPUT_NODATA_CHECK##" ), inputNoDataCheck.isEmpty() ? QStringLiteral( "false" ) : inputNoDataCheck.join( QLatin1String( " || " ) ) );
     programTemplate = programTemplate.replace( QLatin1String( "##INPUT_DESC##" ), inputDesc.join( '\n' ) );
     programTemplate = programTemplate.replace( QLatin1String( "##INPUT##" ), !inputArgs.isEmpty() ? ( inputArgs.join( ',' ).append( ',' ) ) : QChar( ' ' ) );
     programTemplate = programTemplate.replace( QLatin1String( "##EXPRESSION##" ), cExpression );
     programTemplate = programTemplate.replace( QLatin1String( "##EXPRESSION_ORIGINAL##" ), calcNode->toString( ) );
 
-    // qDebug() << programTemplate;
+    //qDebug() << programTemplate;
 
     // Create a program from the kernel source
     cl::Program program( QgsOpenClUtils::buildProgram( programTemplate, QgsOpenClUtils::ExceptionBehavior::Throw ) );
@@ -628,6 +664,8 @@ QgsRasterCalculator::Result QgsRasterCalculator::processCalculationGPU( std::uni
 
     inputBuffers.clear();
 
+    GDALComputeRasterStatistics( outputRasterBand, true, nullptr, nullptr, nullptr, nullptr, GdalProgressCallback, feedback );
+
   }
   catch ( cl::Error &e )
   {
@@ -702,7 +740,7 @@ QVector<QgsRasterCalculatorEntry> QgsRasterCalculatorEntry::rasterEntries()
     while ( true )
     {
       bool unique( true );
-      for ( const auto &ref : qgis::as_const( availableEntries ) )
+      for ( const auto &ref : std::as_const( availableEntries ) )
       {
         // Safety belt
         if ( !( entry.raster && ref.raster ) )
@@ -735,7 +773,7 @@ QVector<QgsRasterCalculatorEntry> QgsRasterCalculatorEntry::rasterEntries()
   for ( ; layerIt != layers.constEnd(); ++layerIt )
   {
     QgsRasterLayer *rlayer = qobject_cast<QgsRasterLayer *>( layerIt.value() );
-    if ( rlayer && rlayer->dataProvider() && rlayer->providerType() == QLatin1String( "gdal" ) )
+    if ( rlayer && rlayer->dataProvider() && ( rlayer->dataProvider()->capabilities() & QgsRasterDataProvider::Size ) )
     {
       //get number of bands
       for ( int i = 0; i < rlayer->bandCount(); ++i )
