@@ -27,7 +27,7 @@
 #include "qgsvectorlayer.h"
 #include "qgssnapindicator.h"
 #include "qgssnappingconfig.h"
-#include "qgssettings.h"
+#include "qgssettingsregistrycore.h"
 #include "qgisapp.h"
 #include "qgsmapmouseevent.h"
 #include "qgslogger.h"
@@ -35,7 +35,7 @@
 
 QgsMapToolOffsetCurve::QgsMapToolOffsetCurve( QgsMapCanvas *canvas )
   : QgsMapToolEdit( canvas )
-  , mSnapIndicator( qgis::make_unique< QgsSnapIndicator >( canvas ) )
+  , mSnapIndicator( std::make_unique< QgsSnapIndicator >( canvas ) )
 {
   mToolName = tr( "Map tool offset curve" );
 }
@@ -103,8 +103,8 @@ void QgsMapToolOffsetCurve::canvasReleaseEvent( QgsMapMouseEvent *e )
         mModifiedFeature = fet.id();
         createUserInputWidget();
 
-        bool hasZ = QgsWkbTypes::hasZ( mSourceLayer->wkbType() );
-        bool hasM = QgsWkbTypes::hasZ( mSourceLayer->wkbType() );
+        const bool hasZ = QgsWkbTypes::hasZ( mSourceLayer->wkbType() );
+        const bool hasM = QgsWkbTypes::hasZ( mSourceLayer->wkbType() );
         if ( hasZ || hasM )
         {
           emit messageEmitted( QStringLiteral( "layer %1 has %2%3%4 geometry. %2%3%4 values be set to 0 when using offset tool." )
@@ -112,7 +112,7 @@ void QgsMapToolOffsetCurve::canvasReleaseEvent( QgsMapMouseEvent *e )
                                      hasZ ? QStringLiteral( "Z" ) : QString(),
                                      hasZ && hasM ? QStringLiteral( "/" ) : QString(),
                                      hasM ? QStringLiteral( "M" ) : QString() )
-                               , Qgis::Warning );
+                               , Qgis::MessageLevel::Warning );
         }
       }
     }
@@ -126,7 +126,7 @@ void QgsMapToolOffsetCurve::canvasReleaseEvent( QgsMapMouseEvent *e )
   else
   {
     // second click - apply changes
-    double offset = calculateOffset( e->snapPoint() );
+    const double offset = calculateOffset( e->snapPoint() );
     applyOffset( offset, e->modifiers() );
   }
 }
@@ -161,11 +161,11 @@ void QgsMapToolOffsetCurve::applyOffset( double offset, Qt::KeyboardModifiers mo
   {
     QgsGeometry geometry;
     int partIndex = 0;
-    QgsWkbTypes::Type geomType = mOriginalGeometry.wkbType();
+    const QgsWkbTypes::Type geomType = mOriginalGeometry.wkbType();
     if ( QgsWkbTypes::geometryType( geomType ) == QgsWkbTypes::LineGeometry )
     {
       QgsMultiPolylineXY newMultiLine;
-      QgsMultiPolylineXY multiLine = mOriginalGeometry.asMultiPolyline();
+      const QgsMultiPolylineXY multiLine = mOriginalGeometry.asMultiPolyline();
       QgsMultiPolylineXY::const_iterator it = multiLine.constBegin();
       for ( ; it != multiLine.constEnd(); ++it )
       {
@@ -289,7 +289,7 @@ void QgsMapToolOffsetCurve::applyOffset( double offset, Qt::KeyboardModifiers mo
         {
           if ( ringIndex == mModifiedRing )
           {
-            QgsMultiPolygonXY ringParts = mModifiedGeometry.asMultiPolygon();
+            const QgsMultiPolygonXY ringParts = mModifiedGeometry.asMultiPolygon();
             QgsPolygonXY newRings;
             QgsMultiPolygonXY::const_iterator ringIt = ringParts.constBegin();
             for ( ; ringIt != ringParts.constEnd(); ++ringIt )
@@ -334,7 +334,7 @@ void QgsMapToolOffsetCurve::applyOffset( double offset, Qt::KeyboardModifiers mo
 
   if ( !mModifiedGeometry.isGeosValid() )
   {
-    emit messageEmitted( tr( "Generated geometry is not valid." ), Qgis::Critical );
+    emit messageEmitted( tr( "Generated geometry is not valid." ), Qgis::MessageLevel::Critical );
     // no cancel, continue editing.
     return;
   }
@@ -352,7 +352,7 @@ void QgsMapToolOffsetCurve::applyOffset( double offset, Qt::KeyboardModifiers mo
   }
   else
   {
-    QgsCoordinateTransform ct( mSourceLayer->crs(), destLayer->crs(), QgsProject::instance() );
+    const QgsCoordinateTransform ct( mSourceLayer->crs(), destLayer->crs(), QgsProject::instance() );
     try
     {
       QgsGeometry g = mModifiedGeometry;
@@ -363,13 +363,13 @@ void QgsMapToolOffsetCurve::applyOffset( double offset, Qt::KeyboardModifiers mo
 
       // auto convert source feature attributes to destination attributes, make geometry compatible
       // note that this may result in multiple features, e.g. if inserting multipart feature into single-part layer
-      QgsFeatureList features = QgsVectorLayerUtils::makeFeatureCompatible( f, destLayer );
+      const QgsFeatureList features = QgsVectorLayerUtils::makeFeatureCompatible( f, destLayer );
       for ( const QgsFeature &feature : features )
       {
         QgsAttributeMap attrs;
         for ( int idx = 0; idx < destLayer->fields().count(); ++idx )
         {
-          if ( !feature.attribute( idx ).isNull() )
+          if ( !QgsVariantUtils::isNull( feature.attribute( idx ) ) )
             attrs[idx] = feature.attribute( idx );
         }
 
@@ -393,7 +393,7 @@ void QgsMapToolOffsetCurve::applyOffset( double offset, Qt::KeyboardModifiers mo
   else
   {
     destLayer->destroyEditCommand();
-    emit messageEmitted( QStringLiteral( "Could not apply offset" ), Qgis::Critical );
+    emit messageEmitted( QStringLiteral( "Could not apply offset" ), Qgis::MessageLevel::Critical );
   }
 
   deleteRubberBandAndGeometry();
@@ -415,7 +415,7 @@ double QgsMapToolOffsetCurve::calculateOffset( const QgsPointXY &mapPoint )
   if ( mSourceLayer )
   {
     //get offset from current position rectangular to feature
-    QgsPointXY layerCoords = toLayerCoordinates( mSourceLayer, mapPoint );
+    const QgsPointXY layerCoords = toLayerCoordinates( mSourceLayer, mapPoint );
 
     QgsPointXY minDistPoint;
     int beforeVertex;
@@ -454,10 +454,10 @@ void QgsMapToolOffsetCurve::canvasMoveEvent( QgsMapMouseEvent *e )
 
   mGeometryModified = true;
 
-  QgsPointXY mapPoint = e->snapPoint();
+  const QgsPointXY mapPoint = e->snapPoint();
   mSnapIndicator->setMatch( e->mapPointMatch() );
 
-  double offset = calculateOffset( mapPoint );
+  const double offset = calculateOffset( mapPoint );
 
   if ( mUserInputWidget )
   {
@@ -486,14 +486,14 @@ void QgsMapToolOffsetCurve::prepareGeometry( const QgsPointLocator::Match &match
   mModifiedRing = -1;
 
   //assign feature part by vertex number (snap to vertex) or by before vertex number (snap to segment)
-  QgsGeometry geom = snappedFeature.geometry();
+  const QgsGeometry geom = snappedFeature.geometry();
   if ( geom.isNull() )
   {
     return;
   }
   mOriginalGeometry = geom;
 
-  QgsWkbTypes::Type geomType = geom.wkbType();
+  const QgsWkbTypes::Type geomType = geom.wkbType();
   if ( QgsWkbTypes::geometryType( geomType ) == QgsWkbTypes::LineGeometry )
   {
     if ( !geom.isMultipart() )
@@ -502,12 +502,12 @@ void QgsMapToolOffsetCurve::prepareGeometry( const QgsPointLocator::Match &match
     }
     else
     {
-      int vertex = match.vertexIndex();
+      const int vertex = match.vertexIndex();
       QgsVertexId vertexId;
       geom.vertexIdFromVertexNr( vertex, vertexId );
       mModifiedPart = vertexId.part;
 
-      QgsMultiPolylineXY multiLine = geom.asMultiPolyline();
+      const QgsMultiPolylineXY multiLine = geom.asMultiPolyline();
       mManipulatedGeometry = QgsGeometry::fromPolylineXY( multiLine.at( mModifiedPart ) );
     }
   }
@@ -526,7 +526,7 @@ void QgsMapToolOffsetCurve::prepareGeometry( const QgsPointLocator::Match &match
         for ( int part = 0; part < mpolygon.count(); part++ ) // go through the polygons
         {
           const QgsPolygonXY &polygon = mpolygon[part];
-          QgsGeometry partGeo = QgsGeometry::fromPolygonXY( polygon );
+          const QgsGeometry partGeo = QgsGeometry::fromPolygonXY( polygon );
           const QgsPointXY layerCoords = match.point();
           if ( partGeo.contains( &layerCoords ) )
           {
@@ -538,14 +538,14 @@ void QgsMapToolOffsetCurve::prepareGeometry( const QgsPointLocator::Match &match
     }
     else if ( match.hasEdge() || match.hasVertex() )
     {
-      int vertex = match.vertexIndex();
+      const int vertex = match.vertexIndex();
       QgsVertexId vertexId;
       geom.vertexIdFromVertexNr( vertex, vertexId );
       QgsDebugMsgLevel( QString::number( vertexId.ring ), 2 );
 
       if ( !geom.isMultipart() )
       {
-        QgsPolygonXY poly = geom.asPolygon();
+        const QgsPolygonXY poly = geom.asPolygon();
         // if has rings
         if ( poly.count() > 0 )
         {
@@ -562,7 +562,7 @@ void QgsMapToolOffsetCurve::prepareGeometry( const QgsPointLocator::Match &match
       {
         mModifiedPart = vertexId.part;
         // get part, get ring
-        QgsMultiPolygonXY multiPoly = geom.asMultiPolygon();
+        const QgsMultiPolygonXY multiPoly = geom.asMultiPolygon();
         // if has rings
         if ( multiPoly.at( mModifiedPart ).count() > 0 )
         {
@@ -628,11 +628,10 @@ void QgsMapToolOffsetCurve::updateGeometryAndRubberBand( double offset )
   }
 
   QgsGeometry offsetGeom;
-  QgsSettings s;
-  QgsGeometry::JoinStyle joinStyle = s.enumValue( QStringLiteral( "/qgis/digitizing/offset_join_style" ),  QgsGeometry::JoinStyleRound );
-  int quadSegments = s.value( QStringLiteral( "/qgis/digitizing/offset_quad_seg" ), 8 ).toInt();
-  double miterLimit = s.value( QStringLiteral( "/qgis/digitizing/offset_miter_limit" ), 5.0 ).toDouble();
-  QgsGeometry::EndCapStyle capStyle = s.enumValue( QStringLiteral( "/qgis/digitizing/offset_cap_style" ),  QgsGeometry::CapRound );
+  const Qgis::JoinStyle joinStyle = QgsSettingsRegistryCore::settingsDigitizingOffsetJoinStyle.value();
+  const int quadSegments = QgsSettingsRegistryCore::settingsDigitizingOffsetQuadSeg.value();
+  const double miterLimit = QgsSettingsRegistryCore::settingsDigitizingOffsetMiterLimit.value();
+  const Qgis::EndCapStyle capStyle = QgsSettingsRegistryCore::settingsDigitizingOffsetCapStyle.value();
 
 
   if ( QgsWkbTypes::geometryType( mOriginalGeometry.wkbType() ) == QgsWkbTypes::LineGeometry )
@@ -651,7 +650,7 @@ void QgsMapToolOffsetCurve::updateGeometryAndRubberBand( double offset )
     mSourceLayer = nullptr;
     mGeometryModified = false;
     emit messageDiscarded();
-    emit messageEmitted( tr( "Creating offset geometry failed: %1" ).arg( offsetGeom.lastError() ), Qgis::Critical );
+    emit messageEmitted( tr( "Creating offset geometry failed: %1" ).arg( offsetGeom.lastError() ), Qgis::MessageLevel::Critical );
   }
   else
   {
@@ -673,40 +672,39 @@ QgsOffsetUserWidget::QgsOffsetUserWidget( QWidget *parent )
   mOffsetSpinBox->setClearValue( 0.0 );
 
   // fill comboboxes
-  mJoinStyleComboBox->addItem( tr( "Round" ), QgsGeometry::JoinStyleRound );
-  mJoinStyleComboBox->addItem( tr( "Miter" ), QgsGeometry::JoinStyleMiter );
-  mJoinStyleComboBox->addItem( tr( "Bevel" ), QgsGeometry::JoinStyleBevel );
-  mCapStyleComboBox->addItem( tr( "Round" ), QgsGeometry::CapRound );
-  mCapStyleComboBox->addItem( tr( "Flat" ), QgsGeometry::CapFlat );
-  mCapStyleComboBox->addItem( tr( "Square" ), QgsGeometry::CapSquare );
+  mJoinStyleComboBox->addItem( tr( "Round" ), static_cast< int >( Qgis::JoinStyle::Round ) );
+  mJoinStyleComboBox->addItem( tr( "Miter" ), static_cast< int >( Qgis::JoinStyle::Miter ) );
+  mJoinStyleComboBox->addItem( tr( "Bevel" ), static_cast< int >( Qgis::JoinStyle::Bevel ) );
+  mCapStyleComboBox->addItem( tr( "Round" ), static_cast< int >( Qgis::EndCapStyle::Round ) );
+  mCapStyleComboBox->addItem( tr( "Flat" ), static_cast< int >( Qgis::EndCapStyle::Flat ) );
+  mCapStyleComboBox->addItem( tr( "Square" ), static_cast< int >( Qgis::EndCapStyle::Square ) );
 
-  QgsSettings s;
-  QgsGeometry::JoinStyle joinStyle = s.enumValue( QStringLiteral( "/qgis/digitizing/offset_join_style" ),  QgsGeometry::JoinStyleRound );
-  int quadSegments = s.value( QStringLiteral( "/qgis/digitizing/offset_quad_seg" ), 8 ).toInt();
-  double miterLimit = s.value( QStringLiteral( "/qgis/digitizing/offset_miter_limit" ), 5.0 ).toDouble();
-  QgsGeometry::EndCapStyle capStyle = s.enumValue( QStringLiteral( "/qgis/digitizing/offset_cap_style" ),  QgsGeometry::CapRound );
+  const Qgis::JoinStyle joinStyle = QgsSettingsRegistryCore::settingsDigitizingOffsetJoinStyle.value();
+  const int quadSegments = QgsSettingsRegistryCore::settingsDigitizingOffsetQuadSeg.value();
+  const double miterLimit = QgsSettingsRegistryCore::settingsDigitizingOffsetMiterLimit.value();
+  const Qgis::EndCapStyle capStyle = QgsSettingsRegistryCore::settingsDigitizingOffsetCapStyle.value();
 
-  mJoinStyleComboBox->setCurrentIndex( mJoinStyleComboBox->findData( joinStyle ) );
+  mJoinStyleComboBox->setCurrentIndex( mJoinStyleComboBox->findData( static_cast< int >( joinStyle ) ) );
   mQuadrantSpinBox->setValue( quadSegments );
   mQuadrantSpinBox->setClearValue( 8 );
   mMiterLimitSpinBox->setValue( miterLimit );
   mMiterLimitSpinBox->setClearValue( 5.0 );
-  mCapStyleComboBox->setCurrentIndex( mCapStyleComboBox->findData( capStyle ) );
+  mCapStyleComboBox->setCurrentIndex( mCapStyleComboBox->findData( static_cast< int >( capStyle ) ) );
 
   // connect signals
   mOffsetSpinBox->installEventFilter( this );
   connect( mOffsetSpinBox, static_cast < void ( QDoubleSpinBox::* )( double ) > ( &QDoubleSpinBox::valueChanged ), this, &QgsOffsetUserWidget::offsetChanged );
 
-  connect( mJoinStyleComboBox, static_cast < void ( QComboBox::* )( int ) > ( &QComboBox::currentIndexChanged ), this, [ = ] { QgsSettings().setEnumValue( QStringLiteral( "/qgis/digitizing/offset_join_style" ), static_cast< QgsGeometry::JoinStyle >( mJoinStyleComboBox->currentData().toInt() ) ); emit offsetConfigChanged(); } );
-  connect( mQuadrantSpinBox, static_cast < void ( QSpinBox::* )( int ) > ( &QSpinBox::valueChanged ), this, [ = ]( const int quadSegments ) { QgsSettings().setValue( QStringLiteral( "/qgis/digitizing/offset_quad_seg" ), quadSegments ); emit offsetConfigChanged(); } );
-  connect( mMiterLimitSpinBox, static_cast < void ( QDoubleSpinBox::* )( double ) > ( &QDoubleSpinBox::valueChanged ), this, [ = ]( double miterLimit ) { QgsSettings().setValue( QStringLiteral( "/qgis/digitizing/offset_miter_limit" ), miterLimit ); emit offsetConfigChanged(); } );
-  connect( mCapStyleComboBox, static_cast < void ( QComboBox::* )( int ) > ( &QComboBox::currentIndexChanged ), this, [ = ] { QgsSettings().setEnumValue( QStringLiteral( "/qgis/digitizing/offset_cap_style" ), static_cast< QgsGeometry::EndCapStyle >( mCapStyleComboBox->currentData().toInt() ) ); emit offsetConfigChanged(); } );
+  connect( mJoinStyleComboBox, static_cast < void ( QComboBox::* )( int ) > ( &QComboBox::currentIndexChanged ), this, [ = ] { QgsSettingsRegistryCore::settingsDigitizingOffsetJoinStyle.setValue( static_cast< Qgis::JoinStyle >( mJoinStyleComboBox->currentData().toInt() ) ); emit offsetConfigChanged(); } );
+  connect( mQuadrantSpinBox, static_cast < void ( QSpinBox::* )( int ) > ( &QSpinBox::valueChanged ), this, [ = ]( const int quadSegments ) { QgsSettingsRegistryCore::settingsDigitizingOffsetQuadSeg.setValue( quadSegments ); emit offsetConfigChanged(); } );
+  connect( mMiterLimitSpinBox, static_cast < void ( QDoubleSpinBox::* )( double ) > ( &QDoubleSpinBox::valueChanged ), this, [ = ]( double miterLimit ) { QgsSettingsRegistryCore::settingsDigitizingOffsetMiterLimit.setValue( miterLimit ); emit offsetConfigChanged(); } );
+  connect( mCapStyleComboBox, static_cast < void ( QComboBox::* )( int ) > ( &QComboBox::currentIndexChanged ), this, [ = ] { QgsSettingsRegistryCore::settingsDigitizingOffsetCapStyle.setValue( static_cast< Qgis::EndCapStyle >( mCapStyleComboBox->currentData().toInt() ) ); emit offsetConfigChanged(); } );
 
-  bool showAdvanced = s.value( QStringLiteral( "/qgis/digitizing/offset_show_advanced" ), false ).toBool();
+  const bool showAdvanced = QgsSettingsRegistryCore::settingsDigitizingOffsetShowAdvanced.value();
   mShowAdvancedButton->setChecked( showAdvanced );
   mAdvancedConfigWidget->setVisible( showAdvanced );
   connect( mShowAdvancedButton, &QToolButton::clicked, mAdvancedConfigWidget, &QWidget::setVisible );
-  connect( mShowAdvancedButton, &QToolButton::clicked, this, [ = ]( const bool clicked ) {QgsSettings().setValue( QStringLiteral( "/qgis/digitizing/offset_show_advanced" ), clicked );} );
+  connect( mShowAdvancedButton, &QToolButton::clicked, this, [ = ]( const bool clicked ) {QgsSettingsRegistryCore::settingsDigitizingConvertToCurveDistanceTolerance.setValue( clicked );} );
 
   // config focus
   setFocusProxy( mOffsetSpinBox );
