@@ -29,7 +29,7 @@ class QToolButton;
 /**
  * \ingroup gui
  * \class QgsColorWidget
- * A base class for interactive color widgets. Widgets can either allow setting a single component of
+ * \brief A base class for interactive color widgets. Widgets can either allow setting a single component of
  * a color (e.g., the red or green components), or an entire color. The QgsColorWidget also keeps track of
  * any explicitly set hue for the color, so that this information is not lost when the widget is
  * set to a color with an ambiguous hue (e.g., black or white shades).
@@ -176,7 +176,7 @@ class GUI_EXPORT QgsColorWidget : public QWidget
     int hue() const;
 
     /**
-     * Alters a color by modifiying the value of a specific color component
+     * Alters a color by modifying the value of a specific color component
      * \param color color to alter
      * \param component color component to alter
      * \param newValue new value of color component. Values are automatically clipped to a
@@ -205,7 +205,7 @@ class GUI_EXPORT QgsColorWidget : public QWidget
 /**
  * \ingroup gui
  * \class QgsColorWidgetAction
- * An action containing a color widget, which can be embedded into a menu.
+ * \brief An action containing a color widget, which can be embedded into a menu.
  * \see QgsColorWidget
  * \since QGIS 2.14
  */
@@ -281,7 +281,7 @@ class GUI_EXPORT QgsColorWidgetAction: public QWidgetAction
 /**
  * \ingroup gui
  * \class QgsColorWheel
- * A color wheel widget. This widget consists of an outer ring which allows for hue selection, and an
+ * \brief A color wheel widget. This widget consists of an outer ring which allows for hue selection, and an
  * inner rotating triangle which allows for saturation and value selection.
  * \since QGIS 2.5
  */
@@ -323,6 +323,8 @@ class GUI_EXPORT QgsColorWheel : public QgsColorWidget
       Triangle
     };
 
+    bool mIsDragging = false;
+
     /*Margin between outer ring and edge of widget*/
     int mMargin = 4;
 
@@ -374,7 +376,7 @@ class GUI_EXPORT QgsColorWheel : public QgsColorWidget
 /**
  * \ingroup gui
  * \class QgsColorBox
- * A color box widget. This widget consists of a two dimensional rectangle filled with color
+ * \brief A color box widget. This widget consists of a two dimensional rectangle filled with color
  * variations, where a different color component varies along both the horizontal and vertical
  * axis.
  * \since QGIS 2.5
@@ -410,9 +412,12 @@ class GUI_EXPORT QgsColorBox : public QgsColorWidget
     void resizeEvent( QResizeEvent *event ) override;
     void mouseMoveEvent( QMouseEvent *event ) override;
     void mousePressEvent( QMouseEvent *event ) override;
+    void mouseReleaseEvent( QMouseEvent *event ) override;
 
   private:
 
+    bool mIsDragging = false;
+
     /*Margin between outer ring and edge of widget*/
     int mMargin = 2;
 
@@ -471,7 +476,7 @@ class GUI_EXPORT QgsColorBox : public QgsColorWidget
 /**
  * \ingroup gui
  * \class QgsColorRampWidget
- * A color ramp widget. This widget consists of an interactive box filled with a color which varies along
+ * \brief A color ramp widget. This widget consists of an interactive box filled with a color which varies along
  * its length by a single color component (e.g., varying saturation from 0 to 100%).
  * \since QGIS 2.5
  */
@@ -565,10 +570,13 @@ class GUI_EXPORT QgsColorRampWidget : public QgsColorWidget
     void mouseMoveEvent( QMouseEvent *event ) override;
     void wheelEvent( QWheelEvent *event ) override;
     void mousePressEvent( QMouseEvent *event ) override;
+    void mouseReleaseEvent( QMouseEvent *event ) override;
     void keyPressEvent( QKeyEvent *event ) override;
 
   private:
 
+    bool mIsDragging = false;
+
     /*Orientation for ramp*/
     Orientation mOrientation;
 
@@ -596,7 +604,7 @@ class GUI_EXPORT QgsColorRampWidget : public QgsColorWidget
 /**
  * \ingroup gui
  * \class QgsColorSliderWidget
- * A composite horizontal color ramp widget and associated spinbox for manual value entry.
+ * \brief A composite horizontal color ramp widget and associated spinbox for manual value entry.
  * \since QGIS 2.5
  */
 
@@ -665,7 +673,7 @@ class GUI_EXPORT QgsColorSliderWidget : public QgsColorWidget
 /**
  * \ingroup gui
  * \class QgsColorTextWidget
- * A line edit widget which displays colors as text and accepts string representations
+ * \brief A line edit widget which displays colors as text and accepts string representations
  * of colors.
  * \since QGIS 2.5
  */
@@ -696,6 +704,13 @@ class GUI_EXPORT QgsColorTextWidget : public QgsColorWidget
 
     void setColor( const QColor &color, bool emitSignals = false ) override;
 
+    /**
+     * Sets whether opacity modification (transparency) is permitted.
+     * Defaults to TRUE.
+     * \param allowOpacity set to FALSE to disable opacity modification
+     */
+    void setAllowOpacity( bool allowOpacity );
+
   protected:
     void resizeEvent( QResizeEvent *event ) override;
 
@@ -709,6 +724,8 @@ class GUI_EXPORT QgsColorTextWidget : public QgsColorWidget
     /*Display format for colors*/
     ColorTextFormat mFormat = QgsColorTextWidget::HexRgb;
 
+    bool mAllowAlpha = true;
+
     /**
      * Updates the text based on the current color
      */
@@ -731,7 +748,7 @@ class GUI_EXPORT QgsColorTextWidget : public QgsColorWidget
 /**
  * \ingroup gui
  * \class QgsColorPreviewWidget
- * A preview box which displays one or two colors as swatches.
+ * \brief A preview box which displays one or two colors as swatches.
  * \since QGIS 2.5
  */
 
