@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """QGIS Unit tests for field formatters.
 
 .. note:: This program is free software; you can redistribute it and/or modify
@@ -10,34 +9,53 @@ __author__ = 'Matthias Kuhn'
 __date__ = '05/12/2016'
 __copyright__ = 'Copyright 2016, The QGIS Project'
 
-
-import tempfile
 import os
+import tempfile
 
 import qgis  # NOQA
-
-from qgis.core import (QgsFeature, QgsProject, QgsRelation, QgsVectorLayer,
-                       QgsValueMapFieldFormatter, QgsValueRelationFieldFormatter,
-                       QgsRelationReferenceFieldFormatter, QgsRangeFieldFormatter,
-                       QgsCheckBoxFieldFormatter, QgsFallbackFieldFormatter,
-                       QgsSettings, QgsGeometry, QgsPointXY, QgsVectorFileWriter)
-
-from qgis.PyQt.QtCore import QCoreApplication, QLocale, QVariant
+from qgis.PyQt.QtCore import (
+    QCoreApplication,
+    QLocale,
+    QVariant,
+    QDate,
+    QTime,
+    QDateTime,
+    Qt
+)
+from qgis.core import (
+    QgsApplication,
+    QgsFeature,
+    QgsProject,
+    QgsRelation,
+    QgsVectorLayer,
+    QgsValueMapFieldFormatter,
+    QgsValueRelationFieldFormatter,
+    QgsRelationReferenceFieldFormatter,
+    QgsRangeFieldFormatter,
+    QgsCheckBoxFieldFormatter,
+    QgsFallbackFieldFormatter,
+    QgsDateTimeFieldFormatter,
+    QgsSettings,
+    QgsGeometry,
+    QgsPointXY,
+    QgsVectorFileWriter
+)
 from qgis.testing import start_app, unittest
 from qgis.utils import spatialite_connect
+
 from utilities import writeShape
 
 start_app()
 
 
 class TestQgsValueMapFieldFormatter(unittest.TestCase):
-
     VALUEMAP_NULL_TEXT = "{2839923C-8B7D-419E-B84B-CA2FE9B80EC7}"
 
     def test_representValue(self):
         QgsSettings().setValue("qgis/nullValue", "NULL")
-        layer = QgsVectorLayer("none?field=number1:integer&field=number2:double&field=text1:string&field=number3:integer&field=number4:double&field=text2:string",
-                               "layer", "memory")
+        layer = QgsVectorLayer(
+            "none?field=number1:integer&field=number2:double&field=text1:string&field=number3:integer&field=number4:double&field=text2:string",
+            "layer", "memory")
         self.assertTrue(layer.isValid())
         QgsProject.instance().addMapLayer(layer)
         f = QgsFeature()
@@ -86,7 +104,6 @@ class TestQgsValueMapFieldFormatter(unittest.TestCase):
 class TestQgsValueRelationFieldFormatter(unittest.TestCase):
 
     def test_representValue(self):
-
         first_layer = QgsVectorLayer("none?field=foreign_key:integer",
                                      "first_layer", "memory")
         self.assertTrue(first_layer.isValid())
@@ -130,7 +147,6 @@ class TestQgsValueRelationFieldFormatter(unittest.TestCase):
         QgsProject.instance().removeMapLayer(second_layer.id())
 
     def test_valueToStringList(self):
-
         def _test(a, b):
             self.assertEqual(QgsValueRelationFieldFormatter.valueToStringList(a), b)
 
@@ -142,11 +158,12 @@ class TestQgsValueRelationFieldFormatter(unittest.TestCase):
         _test('{1,2,3}', ["1", "2", "3"])
         _test('{"1","2","3"}', ["1", "2", "3"])
         _test('["1","2","3"]', ["1", "2", "3"])
-        _test(r'["a string,comma","a string\"quote", "another string[]"]', ['a string,comma', 'a string"quote', 'another string[]'])
+        _test(r'["a string,comma","a string\"quote", "another string[]"]',
+              ['a string,comma', 'a string"quote', 'another string[]'])
 
     def test_expressionRequiresFormScope(self):
-
-        res = list(QgsValueRelationFieldFormatter.expressionFormAttributes("current_value('ONE') AND current_value('TWO')"))
+        res = list(
+            QgsValueRelationFieldFormatter.expressionFormAttributes("current_value('ONE') AND current_value('TWO')"))
         res = sorted(res)
         self.assertEqual(res, ['ONE', 'TWO'])
 
@@ -173,17 +190,19 @@ class TestQgsValueRelationFieldFormatter(unittest.TestCase):
         self.assertTrue(QgsValueRelationFieldFormatter.expressionIsUsable("current_geometry", f))
         self.assertFalse(QgsValueRelationFieldFormatter.expressionIsUsable("current_value ( 'TWO' )", f))
         self.assertTrue(QgsValueRelationFieldFormatter.expressionIsUsable("current_value ( 'pkid' )", f))
-        self.assertTrue(QgsValueRelationFieldFormatter.expressionIsUsable("@current_geometry current_value ( 'pkid' )", f))
+        self.assertTrue(
+            QgsValueRelationFieldFormatter.expressionIsUsable("@current_geometry current_value ( 'pkid' )", f))
 
         QgsProject.instance().removeMapLayer(layer.id())
 
     def test_expressionRequiresParentFormScope(self):
-
-        res = list(QgsValueRelationFieldFormatter.expressionFormAttributes("current_value('ONE') AND current_parent_value('TWO')"))
+        res = list(QgsValueRelationFieldFormatter.expressionFormAttributes(
+            "current_value('ONE') AND current_parent_value('TWO')"))
         res = sorted(res)
         self.assertEqual(res, ['ONE'])
 
-        res = list(QgsValueRelationFieldFormatter.expressionParentFormAttributes("current_value('ONE') AND current_parent_value('TWO')"))
+        res = list(QgsValueRelationFieldFormatter.expressionParentFormAttributes(
+            "current_value('ONE') AND current_parent_value('TWO')"))
         res = sorted(res)
         self.assertEqual(res, ['TWO'])
 
@@ -192,19 +211,20 @@ class TestQgsValueRelationFieldFormatter(unittest.TestCase):
 
         self.assertFalse(QgsValueRelationFieldFormatter.expressionRequiresParentFormScope(""))
         self.assertTrue(QgsValueRelationFieldFormatter.expressionRequiresParentFormScope("current_parent_value('TWO')"))
-        self.assertTrue(QgsValueRelationFieldFormatter.expressionRequiresParentFormScope("current_parent_value ( 'TWO' )"))
+        self.assertTrue(
+            QgsValueRelationFieldFormatter.expressionRequiresParentFormScope("current_parent_value ( 'TWO' )"))
         self.assertTrue(QgsValueRelationFieldFormatter.expressionRequiresParentFormScope("@current_parent_geometry"))
 
 
 class TestQgsRelationReferenceFieldFormatter(unittest.TestCase):
 
     def test_representValue(self):
-
         first_layer = QgsVectorLayer("none?field=foreign_key:integer",
                                      "first_layer", "memory")
         self.assertTrue(first_layer.isValid())
         second_layer = QgsVectorLayer("none?field=pkid:integer&field=decoded:string",
                                       "second_layer", "memory")
+        second_layer.setDisplayExpression('pkid')
         self.assertTrue(second_layer.isValid())
         QgsProject.instance().addMapLayers([first_layer, second_layer])
         f = QgsFeature()
@@ -243,10 +263,10 @@ class TestQgsRelationReferenceFieldFormatter(unittest.TestCase):
         second_layer.setDisplayExpression('decoded')
         self.assertEqual(fieldFormatter.representValue(first_layer, 0, config, None, '123'), '123')
 
-        # No display expression
+        # No display expression - will default internally to the decoded string
         config = {'Relation': rel.id()}
         second_layer.setDisplayExpression(None)
-        self.assertEqual(fieldFormatter.representValue(first_layer, 0, config, None, '123'), '123')
+        self.assertEqual(fieldFormatter.representValue(first_layer, 0, config, None, '123'), 'decoded_val')
 
         # Invalid display expression
         config = {'Relation': rel.id()}
@@ -303,7 +323,6 @@ class TestQgsRangeFieldFormatter(unittest.TestCase):
         QLocale.setDefault(QLocale(QLocale.English))
 
     def test_representValue(self):
-
         layer = QgsVectorLayer("point?field=int:integer&field=double:double&field=long:long",
                                "layer", "memory")
         self.assertTrue(layer.isValid())
@@ -314,11 +333,13 @@ class TestQgsRangeFieldFormatter(unittest.TestCase):
         # Precision is ignored for integers and longlongs
         self.assertEqual(fieldFormatter.representValue(layer, 0, {'Precision': 1}, None, '123'), '123')
         self.assertEqual(fieldFormatter.representValue(layer, 0, {'Precision': 1}, None, '123000'), '123,000')
-        self.assertEqual(fieldFormatter.representValue(layer, 0, {'Precision': 1}, None, '9999999'), '9,999,999')  # no scientific notation for integers!
+        self.assertEqual(fieldFormatter.representValue(layer, 0, {'Precision': 1}, None, '9999999'),
+                         '9,999,999')  # no scientific notation for integers!
         self.assertEqual(fieldFormatter.representValue(layer, 0, {'Precision': 1}, None, None), 'NULL')
         self.assertEqual(fieldFormatter.representValue(layer, 2, {'Precision': 1}, None, '123'), '123')
         self.assertEqual(fieldFormatter.representValue(layer, 2, {'Precision': 1}, None, '123000'), '123,000')
-        self.assertEqual(fieldFormatter.representValue(layer, 2, {'Precision': 1}, None, '9999999'), '9,999,999')  # no scientific notation for long longs!
+        self.assertEqual(fieldFormatter.representValue(layer, 2, {'Precision': 1}, None, '9999999'),
+                         '9,999,999')  # no scientific notation for long longs!
         self.assertEqual(fieldFormatter.representValue(layer, 2, {'Precision': 1}, None, None), 'NULL')
 
         self.assertEqual(fieldFormatter.representValue(layer, 1, {'Precision': 1}, None, None), 'NULL')
@@ -347,7 +368,8 @@ class TestQgsRangeFieldFormatter(unittest.TestCase):
                          '9.999.999')  # scientific notation for integers!
         self.assertEqual(fieldFormatter.representValue(layer, 2, {'Precision': 1}, None, '123'), '123')
         self.assertEqual(fieldFormatter.representValue(layer, 2, {'Precision': 1}, None, '123000'), '123.000')
-        self.assertEqual(fieldFormatter.representValue(layer, 2, {'Precision': 1}, None, '9999999'), '9.999.999')  # scientific notation for long longs!
+        self.assertEqual(fieldFormatter.representValue(layer, 2, {'Precision': 1}, None, '9999999'),
+                         '9.999.999')  # scientific notation for long longs!
         self.assertEqual(fieldFormatter.representValue(layer, 2, {'Precision': 1}, None, None), 'NULL')
 
         self.assertEqual(fieldFormatter.representValue(layer, 1, {'Precision': 2}, None, None), 'NULL')
@@ -376,7 +398,8 @@ class TestQgsRangeFieldFormatter(unittest.TestCase):
                          '9999999')  # scientific notation for integers!
         self.assertEqual(fieldFormatter.representValue(layer, 2, {'Precision': 1}, None, '123'), '123')
         self.assertEqual(fieldFormatter.representValue(layer, 2, {'Precision': 1}, None, '123000'), '123000')
-        self.assertEqual(fieldFormatter.representValue(layer, 2, {'Precision': 1}, None, '9999999'), '9999999')  # scientific notation for long longs!
+        self.assertEqual(fieldFormatter.representValue(layer, 2, {'Precision': 1}, None, '9999999'),
+                         '9999999')  # scientific notation for long longs!
         self.assertEqual(fieldFormatter.representValue(layer, 1, {'Precision': 2}, None, '123000'), '123000.00')
 
         QgsProject.instance().removeAllMapLayers()
@@ -561,10 +584,14 @@ class TestQgsFallbackFieldFormatter(unittest.TestCase):
             self.assertEqual(fieldFormatter.representValue(layer, 3 + offset, {}, None, None), 'NULL')
 
             # Check NULLs (this is what happens in real life inside QGIS)
-            self.assertEqual(fieldFormatter.representValue(layer, 0 + offset, {}, None, QVariant(QVariant.String)), 'NULL')
-            self.assertEqual(fieldFormatter.representValue(layer, 1 + offset, {}, None, QVariant(QVariant.String)), 'NULL')
-            self.assertEqual(fieldFormatter.representValue(layer, 2 + offset, {}, None, QVariant(QVariant.String)), 'NULL')
-            self.assertEqual(fieldFormatter.representValue(layer, 3 + offset, {}, None, QVariant(QVariant.String)), 'NULL')
+            self.assertEqual(fieldFormatter.representValue(layer, 0 + offset, {}, None, QVariant(QVariant.String)),
+                             'NULL')
+            self.assertEqual(fieldFormatter.representValue(layer, 1 + offset, {}, None, QVariant(QVariant.String)),
+                             'NULL')
+            self.assertEqual(fieldFormatter.representValue(layer, 2 + offset, {}, None, QVariant(QVariant.String)),
+                             'NULL')
+            self.assertEqual(fieldFormatter.representValue(layer, 3 + offset, {}, None, QVariant(QVariant.String)),
+                             'NULL')
 
         memory_layer = QgsVectorLayer("point?field=int:integer&field=double:double&field=long:long&field=string:string",
                                       "layer", "memory")
@@ -637,5 +664,81 @@ class TestQgsFallbackFieldFormatter(unittest.TestCase):
         self.assertEqual(fieldFormatter.representValue(vl, 3, {}, None, 5), "")
 
 
+class TestQgsDateTimeFieldFormatter(unittest.TestCase):
+
+    @classmethod
+    def setUpClass(cls):
+        """Run before all tests"""
+        QCoreApplication.setOrganizationName("QGIS_Test")
+        QCoreApplication.setOrganizationDomain("QGIS_TestQgsDateTimeFieldFormatter.com")
+        QCoreApplication.setApplicationName("QGIS_TestQgsDateTimeFieldFormatter")
+        QgsSettings().clear()
+        QLocale.setDefault(QLocale(QLocale.English))
+        start_app()
+
+    @classmethod
+    def tearDownClass(cls):
+        """Reset locale"""
+        QgsApplication.setLocale(QLocale(QLocale.English))
+
+    def test_representValue(self):
+        layer = QgsVectorLayer("point?field=datetime:datetime&field=date:date&field=time:time",
+                               "layer", "memory")
+        self.assertTrue(layer.isValid())
+        QgsProject.instance().addMapLayers([layer])
+
+        field_formatter = QgsDateTimeFieldFormatter()
+
+        # if specific display format is set then use that
+        config = {"display_format": "dd/MM/yyyy HH:mm:ss"}
+        self.assertEqual(field_formatter.representValue(layer, 0, config, None,
+                                                        QDateTime(QDate(2020, 3, 4), QTime(12, 13, 14), Qt.UTC)),
+                         '04/03/2020 12:13:14')
+        self.assertEqual(field_formatter.representValue(layer, 0, config, None,
+                                                        QDateTime(QDate(2020, 3, 4), QTime(12, 13, 14), Qt.OffsetFromUTC, 3600)),
+                         '04/03/2020 12:13:14')
+
+        locale_assertions = {
+            QLocale(QLocale.English): {
+                "date_format": 'M/d/yy',
+                "time_format": 'HH:mm:ss',
+                "datetime_format": 'M/d/yy HH:mm:ss',
+                "datetime_utc": '3/4/20 12:13:14 (UTC)',
+                "datetime_utc+1": '3/4/20 12:13:14 (UTC+01:00)'
+            },
+            QLocale(QLocale.Finnish): {
+                "date_format": 'd.M.yyyy',
+                "time_format": 'HH:mm:ss',
+                "datetime_format": 'd.M.yyyy HH:mm:ss',
+                "datetime_utc": '4.3.2020 12:13:14 (UTC)',
+                "datetime_utc+1": '4.3.2020 12:13:14 (UTC+01:00)'
+            },
+        }
+
+        for locale, assertions in locale_assertions.items():
+            QgsApplication.setLocale(locale)
+            field_formatter = QgsDateTimeFieldFormatter()
+
+            self.assertEqual(field_formatter.defaultFormat(QVariant.Date), assertions["date_format"], locale.name())
+            self.assertEqual(field_formatter.defaultFormat(QVariant.Time), assertions["time_format"], locale.name())
+            self.assertEqual(field_formatter.defaultFormat(QVariant.DateTime), assertions["datetime_format"], locale.name())
+
+            # default configuration should show timezone information
+            config = {}
+            self.assertEqual(field_formatter.representValue(layer, 0, config, None,
+                                                            QDateTime(QDate(2020, 3, 4), QTime(12, 13, 14), Qt.UTC)),
+                             assertions["datetime_utc"], locale.name())
+            self.assertEqual(field_formatter.representValue(layer, 0, config, None,
+                                                            QDateTime(QDate(2020, 3, 4), QTime(12, 13, 14), Qt.OffsetFromUTC, 3600)),
+                             assertions["datetime_utc+1"], locale.name())
+            self.assertEqual(field_formatter.representValue(layer, 1, config, None,
+                                                            QDate(2020, 3, 4)),
+                             assertions["datetime_utc"].split(" ")[0], locale.name())
+            config = {"display_format": "HH:mm:s"}
+            self.assertEqual(field_formatter.representValue(layer, 2, config, None,
+                                                            QTime(12, 13, 14)),
+                             assertions["datetime_utc"].split(" ")[1], locale.name())
+
+
 if __name__ == '__main__':
     unittest.main()
