@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """QGIS Unit tests for QgsFieldMapping widget and model.
 
 .. note:: This program is free software; you can redistribute it and/or modify
@@ -11,18 +10,8 @@ __author__ = 'Alessandro Pasotti'
 __date__ = '16/03/2020'
 __copyright__ = 'Copyright 2020, The QGIS Project'
 # This will get replaced with a git SHA1 when you do a git archive
-__revision__ = '4279198632348b51b6985e3c3012256e953c411d'
+__revision__ = '66f4d63ad4f2888134babb55459f8b87369dca46'
 
-from qgis.core import (
-    QgsFields,
-    QgsField,
-    QgsFieldConstraints,
-    QgsProperty
-)
-from qgis.gui import (
-    QgsFieldMappingWidget,
-    QgsFieldMappingModel,
-)
 from qgis.PyQt.Qt import Qt
 from qgis.PyQt.QtCore import (
     QCoreApplication,
@@ -33,6 +22,16 @@ from qgis.PyQt.QtCore import (
 from qgis.PyQt.QtGui import (
     QColor
 )
+from qgis.core import (
+    QgsFields,
+    QgsField,
+    QgsFieldConstraints,
+    QgsProperty
+)
+from qgis.gui import (
+    QgsFieldMappingWidget,
+    QgsFieldMappingModel,
+)
 from qgis.testing import start_app, unittest
 
 
@@ -60,6 +59,7 @@ class TestPyQgsFieldMappingModel(unittest.TestCase):
         f = QgsField('destination_field1', QVariant.Int, 'integer', 10, 8)
         self.assertTrue(destination_fields.append(f))
         f = QgsField('destination_field2', QVariant.String)
+        f.setAlias('my alias')
         self.assertTrue(destination_fields.append(f))
         f = QgsField('destination_field3', QVariant.String)
         self.assertTrue(destination_fields.append(f))
@@ -95,7 +95,7 @@ class TestPyQgsFieldMappingModel(unittest.TestCase):
         self.assertEqual(model.data(model.index(0, 4), Qt.DisplayRole), 8)
 
         self.assertEqual(model.data(model.index(1, 0), Qt.DisplayRole), '"source_field1"')
-        self.assertEqual(model.data(model.index(1, 1), Qt.DisplayRole), 'destination_field2')
+        self.assertEqual(model.data(model.index(1, 1), Qt.DisplayRole), 'destination_field2 (my alias)')
 
         self.assertEqual(model.data(model.index(2, 0), Qt.DisplayRole), QVariant())
         self.assertEqual(model.data(model.index(2, 1), Qt.DisplayRole), 'destination_field3')
@@ -170,7 +170,7 @@ class TestPyQgsFieldMappingModel(unittest.TestCase):
         self.assertEqual(model.data(model.index(0, 0), Qt.DisplayRole), '"source_field2"')
         self.assertEqual(model.data(model.index(0, 1), Qt.DisplayRole), 'destination_field1')
         self.assertEqual(model.data(model.index(1, 0), Qt.DisplayRole), '"source_field1"')
-        self.assertEqual(model.data(model.index(1, 1), Qt.DisplayRole), 'destination_field2')
+        self.assertEqual(model.data(model.index(1, 1), Qt.DisplayRole), 'destination_field2 (my alias)')
         self.assertEqual(model.data(model.index(2, 0), Qt.DisplayRole), '"source_field3"')
         self.assertEqual(model.data(model.index(2, 1), Qt.DisplayRole), 'destination_field3')
 
